@@ -5,56 +5,90 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
+ * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
+ * modify, and redistribute this software under certain conditions.  If    *
+ * you wish to embed Nmap technology into proprietary software, we sell    *
+ * alternative licenses (contact sales@insecure.com).  Dozens of software  *
+ * vendors already license Nmap technology such as host discovery, port    *
+ * scanning, OS detection, version detection, and the Nmap Scripting       *
+ * Engine.                                                                 *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, Insecure.Com LLC grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * linked combinations including the two.                                  *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the special and conditions of the license text as well.       *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, are happy to help.  As mentioned above, we also    *
+ * offer alternative license to integrate Nmap into proprietary            *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * If you received these files with a written license agreement or         *
  * contract stating terms other than the terms above, then that            *
@@ -67,24 +101,24 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
  *                                                                         *
  ***************************************************************************/
 #include "nping.h"
@@ -103,7 +137,7 @@ extern NpingOps o;
 int atoIP(const char *hostname, struct in_addr *dst){
   struct sockaddr_in i;
   unsigned int stlen=0;
-  if ( resolve(hostname, 0, 0, (sockaddr_storage*)&i, (size_t *)&stlen , PF_INET) == 0 )
+  if ( resolve(hostname, 0, (sockaddr_storage*)&i, (size_t *)&stlen , PF_INET) != 0 )
     return OP_FAILURE;
   *dst=i.sin_addr;
   return OP_SUCCESS;
@@ -115,7 +149,7 @@ int atoIP(const char *hostname, struct sockaddr_storage *ss, int family){
     return OP_FAILURE;
   if(family!=AF_INET && family!=AF_INET6)
     return OP_FAILURE;
-  if ( resolve(hostname, 0, 0, ss, &stlen , family) == 0 )
+  if ( resolve(hostname, 0, ss, &stlen , family) != 0 )
     return OP_FAILURE;
   return OP_SUCCESS;
 } /* End of atoIP */
@@ -275,6 +309,31 @@ u16 sockaddr2port(struct sockaddr_in6 *s6){
 }
 
 
+/* Sets the address family member of the supplied sockaddr. */
+int setsockaddrfamily(struct sockaddr_storage *ss, int family){
+  struct sockaddr_in *s4=(struct sockaddr_in *)ss;
+  s4->sin_family=family;
+  return OP_SUCCESS;
+} /* End of setsockaddrfamily() */
+
+
+/* Sets the special INADDR_ANY or in6addr_an constant on the sin_family or
+ * sin6_addr member of the supplied sockaddr. Note that for this to work,
+ * the supplied sockaddr_storage MUST have a correct address family set 
+ * already (sin_family or sin6_family). */
+int setsockaddrany(struct sockaddr_storage *ss){
+  struct sockaddr_in *s4=(struct sockaddr_in *)ss;
+  struct sockaddr_in6 *s6=(struct sockaddr_in6 *)ss;
+  if(s4->sin_family==AF_INET)
+    s4->sin_addr.s_addr=INADDR_ANY;
+  else if(s6->sin6_family==AF_INET6)
+    s6->sin6_addr=in6addr_any;
+  else
+    return OP_FAILURE;
+  return OP_SUCCESS;
+} /* End of setsockaddrany() */
+
+
 /** Returns true if supplied value corresponds to a valid RFC compliant ICMP
  *  Code. Otherwise it returns false.
  *  @warning The fact that a given value matches a standard code does not
@@ -389,7 +448,7 @@ int getPacketStrInfo(const char *proto, const u8 *packet, u32 len, u8 *dstbuff,
   int detail;
 
   if ( dstbuff == NULL || dstlen < 512 )
-    outFatal(QT_3,"safe_ippackethdrinfo() Invalid values supplied.");
+    nping_fatal(QT_3,"safe_ippackethdrinfo() Invalid values supplied.");
 
   if(o.getVerbosity()>=VB_2)
     detail=HIGH_DETAIL;
@@ -398,7 +457,7 @@ int getPacketStrInfo(const char *proto, const u8 *packet, u32 len, u8 *dstbuff,
   else
     detail=LOW_DETAIL;
 
-  if( !strcasecmp(proto, "IP") || !strcasecmp(proto, "IPv4") ){
+  if( !strcasecmp(proto, "IP") || !strcasecmp(proto, "IPv4") || !strcasecmp(proto, "IPv6")){
     b=(char *)ippackethdrinfo(packet, len, detail);
     strncpy((char*)dstbuff, b, dstlen);
     dstbuff[dstlen-1]=0; /* Just to be sure, NULL-terminate the last position*/
@@ -410,9 +469,9 @@ int getPacketStrInfo(const char *proto, const u8 *packet, u32 len, u8 *dstbuff,
     else if ( o.getMode()==UDP )
         return  udppackethdrinfo(packet, len, dstbuff, dstlen, detail, ss_src, ss_dst);
     else
-        outFatal(QT_3, "getPacketStrInfo(): Unable to determinate transport layer protocol");
+        nping_fatal(QT_3, "getPacketStrInfo(): Unable to determinate transport layer protocol");
   }else{
-    outFatal(QT_3, "getPacketStrInfo(): Unkwnown protocol");
+    nping_fatal(QT_3, "getPacketStrInfo(): Unkwnown protocol");
   }
   return OP_SUCCESS;
 } /* getPacketStrInfo() */
@@ -478,7 +537,7 @@ int getNetworkInterfaceName(u32 destination, char *dev){
   struct sockaddr_in dst, src;
   bool result=false;
   if(dev==NULL)
-    outFatal(QT_3, "getNetworkInterfaceName(): NULL value supplied.");
+    nping_fatal(QT_3, "getNetworkInterfaceName(): NULL value supplied.");
   memset(&rnfo, 0, sizeof(struct route_nfo) );
   memset(&dst, 0, sizeof(struct sockaddr_in) );
   memset(&src, 0, sizeof(struct sockaddr_in) );
@@ -502,7 +561,7 @@ int getNetworkInterfaceName(struct sockaddr_storage *dst, char *dev){
   struct sockaddr_storage src;
   bool result=false;
   if(dev==NULL)
-    outFatal(QT_3, "getNetworkInterfaceName(): NULL value supplied.");
+    nping_fatal(QT_3, "getNetworkInterfaceName(): NULL value supplied.");
   memset(&rnfo, 0, sizeof(struct route_nfo) );
   memset(&src, 0, sizeof(struct sockaddr_in) );
   result=route_dst(dst, &rnfo, NULL, NULL); 
@@ -530,13 +589,13 @@ int resolveCached(char *host, struct sockaddr_storage *ss, size_t *sslen, int pf
 
   /* Used for debug. When called with NULL,0x1337, print stats */
   if(host==NULL && pf == 1337){
-	outPrint(DBG_4, "resolveCached(): MISSES: %d,  HITS: %d\n", misses, hits);
+	nping_print(DBG_4, "resolveCached(): MISSES: %d,  HITS: %d\n", misses, hits);
 	return OP_SUCCESS;
   }
 
 
   if( ss==NULL || sslen==NULL || host==NULL)
-	outFatal(QT_3, "resolveCached(): NULL values supplied");
+	nping_fatal(QT_3, "resolveCached(): NULL values supplied");
 	
   /* First we check if we have the host already cached */
   for(int i=0; i<MAX_CACHED_HOSTS && i<cached_count; i++){
@@ -544,16 +603,16 @@ int resolveCached(char *host, struct sockaddr_storage *ss, size_t *sslen, int pf
 		*sslen=archive[i].sslen;
 		memcpy(ss, &(archive[i].ss) , *sslen);
 		hits++;
-		outPrint(DBG_4, "resolveCached(): Cache hit %d for %s\n", hits, host);
+		nping_print(DBG_4, "resolveCached(): Cache hit %d for %s\n", hits, host);
 		return OP_SUCCESS;		
 	}
   }
 
   /* Cache miss */
   misses++;
-  outPrint(DBG_4, "resolveCached(): Cache miss %d for %s\n", misses, host);
+  nping_print(DBG_4, "resolveCached(): Cache miss %d for %s\n", misses, host);
 
-  if( (result=resolve(host, 0, 0, ss, sslen, pf)) == 1 ){
+  if( (result=resolve(host, 0, ss, sslen, pf)) == 0 ){
 	
 	  /* Increment count */
 	  if( cached_count < MAX_CACHED_HOSTS )
@@ -603,7 +662,7 @@ int resolveCached(char *host, struct sockaddr_storage *ss, size_t *sslen, int pf
 	  //return OP_SUCCESS;
 	
   }else{
-		outError(QT_2, "Error resolving %s\n",host);
+		nping_warning(QT_2, "Error resolving %s\n",host);
 		return OP_FAILURE;
   }
 } /* End of resolveCached() */
@@ -624,20 +683,20 @@ struct hostent *gethostbynameCached(char *host){
   int i=0;
 
   if( host==NULL)
-	outFatal(QT_3, "gethostbynameCached(): NULL values supplied");
+	nping_fatal(QT_3, "gethostbynameCached(): NULL values supplied");
 
   /* First we check if we have the host already cached */
   for(i=0; i<MAX_CACHED_HOSTS && i<cached_count; i++){
     if( !strcasecmp( archive[i].hostname , host ) ){ /* Cache hit */
 		hits++;
-		outPrint(DBG_4, "gethostbynameCached(): Cache hit %d for %s", hits, host);
+		nping_print(DBG_4, "gethostbynameCached(): Cache hit %d for %s", hits, host);
 		return  archive[i].h;
 	}
   }
 
   /* Cache miss */
   misses++;
-  outPrint(DBG_4, "gethostbynameCached(): Cache miss %d for %s", misses, host);
+  nping_print(DBG_4, "gethostbynameCached(): Cache miss %d for %s", misses, host);
 
   if( (result=gethostbyname(host) ) != NULL ){
 	
@@ -826,7 +885,7 @@ char *MACtoa(u8 *mac){
 const char *arppackethdrinfo(const u8 *packet, u32 len, int detail){
   static char protoinfo[512];
   if (packet==NULL)
-    outFatal(QT_3, "arppackethdrinfo(): NULL value supplied");
+    nping_fatal(QT_3, "arppackethdrinfo(): NULL value supplied");
   if( len < 28 )
     return "BOGUS!  Packet too short.";
   u16 *htype = (u16 *)packet;
@@ -868,7 +927,7 @@ int arppackethdrinfo(const u8 *packet, u32 len, u8 *dstbuff, u32 dstlen){
   int detail=0;
 
   if ( dstbuff == NULL || dstlen < 512 )
-    outFatal(QT_3,"safe_arppackethdrinfo() Invalid values supplied.");
+    nping_fatal(QT_3,"safe_arppackethdrinfo() Invalid values supplied.");
 
   /* Determine level of detail in packet output from current verbosity level */
   if(o.getVerbosity()>=VB_2)
@@ -1129,14 +1188,14 @@ int send_packet(NpingTarget *target, int rawfd, u8 *pkt, size_t pktLen){
             if(dport!=NULL)
                 s6.sin6_port = *dport;
             else
-                outFatal(QT_3, "send_packet(): Could not determine TCP destination port.");
+                nping_fatal(QT_3, "send_packet(): Could not determine TCP destination port.");
         }
         else if( o.getMode()==UDP){
             dport=getDstPortFromUDPHeader(pkt, pktLen);
             if(dport!=NULL)
                 s6.sin6_port = *dport;
             else
-                outFatal(QT_3, "send_packet(): Could not determine UDP destination port.");
+                nping_fatal(QT_3, "send_packet(): Could not determine UDP destination port.");
         }
         */
 
@@ -1148,11 +1207,21 @@ int send_packet(NpingTarget *target, int rawfd, u8 *pkt, size_t pktLen){
          s6.sin6_port=0;
 
         res = Sendto("send_packet", rawfd, pkt, pktLen, 0, (struct sockaddr *)&s6, (int) sizeof(struct sockaddr_in6));
+        /*Sendto returns errors as -1 according to netutil.cc so lets catch that and return OP_FAILURE*/
+        if (res == -1) return OP_FAILURE;
     }else{ /* IPv4 */
+        struct sockaddr_storage dst;
+        size_t dstlen;
+
+        dstlen = sizeof(dst);
+        target->getTargetSockAddr(&dst, &dstlen);
+        assert(dst.ss_family == AF_INET);
         if( o.issetMTU() == true )
-            res = send_frag_ip_packet(rawfd, NULL, pkt, pktLen, o.getMTU() );
+            res = send_frag_ip_packet(rawfd, NULL, (struct sockaddr_in *) &dst, pkt, pktLen, o.getMTU() );
         else
-            res = send_ip_packet_sd(rawfd, pkt, pktLen);
+            res = send_ip_packet_sd(rawfd, (struct sockaddr_in *) &dst, pkt, pktLen);
+        /*send_ip_packet_sd calls Sendto which returns errors as -1 according to netutil.cc so lets catch that and return OP_FAILURE*/
+        if (res == -1) return OP_FAILURE;
     }
   }
   return OP_SUCCESS;
@@ -1222,36 +1291,6 @@ struct sockaddr_storage *getSrcSockAddrFromIPPacket(u8 *pkt, size_t pktLen){
 } /* End of getSrcSockAddrFromPacket() */
 
 
-struct sockaddr_storage *getDestAddrFromICMPPacket(u8 *pkt, size_t pktLen){
-  static struct sockaddr_storage ss;
-  struct sockaddr_in *s_ip4=(struct sockaddr_in *)&ss;
-  struct ip *i4=(struct ip*)pkt;
-  struct ip *orig_i4=NULL;
-  memset(&ss, 0, sizeof(struct sockaddr_storage));
-
-  if(pkt==NULL || pktLen < 48) /* 48 = First IP hdr + ICMP pkt + Embedded IP hdr */
-    return NULL;
-
-  if( i4->ip_v == 4 ){
-    /* Let's make sure we can trust i4->ip_hl field. We just check if we
-     * have enough bytes to access the DST Addr of the original IP header that
-     * is included in the ICMP packet. */
-    if ( (size_t)(i4->ip_hl*4 + 8 + 16 + 4) > pktLen )
-        return NULL;
-    /* Let's make sure the ICMP pkt contains an IPv4 packet */
-    orig_i4=(struct ip*)( pkt + (8 + i4->ip_hl*4) );
-    if( orig_i4->ip_v != 4 )
-        return NULL;
-    s_ip4->sin_family=AF_INET;    
-    memcpy(&(s_ip4->sin_addr.s_addr), pkt + i4->ip_hl*4 + 8 + 16, 4);
-    return &ss;
-  }
-  else {
-    return NULL;
-  }
-} /* End of getDestAddrFromICMPPacket */
-
-
 
 
 
@@ -1433,21 +1472,21 @@ int obtainRawSocket(){
         break;
         
         case ARP:
-            outError(QT_2,"Warning: createRawSocket() should not be called in ARP mode.");
+            nping_warning(QT_2,"Warning: createRawSocket() should not be called in ARP mode.");
             return 0;
         break;
         
         default:
-            outFatal(QT_3, "createRawSocket(): NpingOps::getMode() does not return a valid mode. Please report this bug.");
+            nping_fatal(QT_3, "createRawSocket(): NpingOps::getMode() does not return a valid mode. Please report this bug.");
         break;
 
     }
     if ((rawipsd = socket(AF_INET6, SOCK_RAW, protocol)) < 0 )
-        outFatal(QT_3,"Couldn't acquire IPv6 raw socket. Are you root?");
+        nping_fatal(QT_3,"Couldn't acquire IPv6 raw socket. Are you root?");
 
   }else{
     if ((rawipsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 )
-        outFatal(QT_3,"Couldn't acquire IPv4 raw socket. Are you root?");
+        nping_fatal(QT_3,"Couldn't acquire IPv4 raw socket. Are you root?");
         /* Tell the kernel we are including our own IP Header (call to 
         * setsockopt passing option IP_HDRINCL)                             */
         sethdrinclude(rawipsd);
@@ -1455,7 +1494,7 @@ int obtainRawSocket(){
 
   /* Allow broadcast addresses */
   if (setsockopt(rawipsd, SOL_SOCKET, SO_BROADCAST, (const char *)&one, sizeof(int)) == -1)
-      outError(QT_2,"Failed to set SO_BROADCAST on raw socket.");
+      nping_warning(QT_2,"Failed to set SO_BROADCAST on raw socket.");
 
   return rawipsd;
 } /* End of obtainRawSocket() */
@@ -1501,20 +1540,20 @@ int getinterfaces_inet6_linux(if6_t *ifbuf, int max_ifaces){
   memset(buffer, 0, sizeof(buffer));
 
   if(ifbuf==NULL || max_ifaces<=0)
-    outFatal(QT_3,"getinterfaces_inet6_linux() NULL values supplied");
+    nping_fatal(QT_3,"getinterfaces_inet6_linux() NULL values supplied");
  
   /* TODO: Do we fatal() or should we just error and return OP_FAILURE? */
-  if ( !fileexistsandisreadable(PATH_PROC_IFINET6) )
-    outFatal(QT_3, "Couldn't get IPv6 interface information. File %s does not exist or you don't have read permissions.", PATH_PROC_IFINET6);
+  if ( !file_is_readable(PATH_PROC_IFINET6) )
+    nping_fatal(QT_3, "Couldn't get IPv6 interface information. File %s does not exist or you don't have read permissions.", PATH_PROC_IFINET6);
   if( (if6file=fopen(PATH_PROC_IFINET6, "r"))==NULL )
-    outFatal(QT_3, "Failed to open %s.", PATH_PROC_IFINET6);
+    nping_fatal(QT_3, "Failed to open %s.", PATH_PROC_IFINET6);
 
   while( fgets(buffer,sizeof(buffer), if6file) ){
 
       if(parsed_ifs>=max_ifaces)
         break;
 
-    outPrint(DBG_4, "Read %s:%d: %s\n", PATH_PROC_IFINET6, ++readlines, buffer);
+    nping_print(DBG_4, "Read %s:%d: %s\n", PATH_PROC_IFINET6, ++readlines, buffer);
 
     /* Check the line has the expected format ********************************/
     /* Some versions of the kernel include colons in the IPv6 address, some
@@ -1695,20 +1734,20 @@ int getroutes_inet6_linux(route6_t *rtbuf, int max_routes){
   memset(buffer, 0, sizeof(buffer));
 
   if(rtbuf==NULL || max_routes<=0)
-    outFatal(QT_3,"getroutes_inet6_linux() NULL values supplied");
+    nping_fatal(QT_3,"getroutes_inet6_linux() NULL values supplied");
  
   /* TODO: Do we fatal() or should we just error and return OP_FAILURE? */
-  if ( !fileexistsandisreadable(PATH_PROC_IPV6ROUTE) )
-    outFatal(QT_3, "Couldn't get IPv6 route information. File %s does not exist or you don't have read permissions.", PATH_PROC_IPV6ROUTE);
+  if ( !file_is_readable(PATH_PROC_IPV6ROUTE) )
+    nping_fatal(QT_3, "Couldn't get IPv6 route information. File %s does not exist or you don't have read permissions.", PATH_PROC_IPV6ROUTE);
   if( (route6file=fopen(PATH_PROC_IPV6ROUTE, "r"))==NULL )
-    outFatal(QT_3, "Failed to open %s.", PATH_PROC_IPV6ROUTE);
+    nping_fatal(QT_3, "Failed to open %s.", PATH_PROC_IPV6ROUTE);
 
   while( fgets(buffer,sizeof(buffer), route6file) ){
 
       if(parsed_routes>=max_routes)
         break;
 
-    outPrint(DBG_4, "Read %s:%d: %s\n",PATH_PROC_IPV6ROUTE, ++readlines, buffer);
+    nping_print(DBG_4, "Read %s:%d: %s\n",PATH_PROC_IPV6ROUTE, ++readlines, buffer);
 
     /* Check the line has the expected format ********************************/
     /* Some versions of the kernel include colons in the IPv6 address, some
