@@ -30,6 +30,12 @@
 --    cause the system to stall through improper use.
 --  - Of course the os and io library can cause the system to also break.
 
+local _VERSION = _VERSION;
+local MAJOR, MINOR = assert(_VERSION:match "^Lua (%d+).(%d+)$");
+if tonumber(MAJOR.."."..MINOR) < 5.2 then
+  error "NSE requires Lua 5.2 or newer. It looks like you're using an older version of nmap."
+end
+
 local NAME = "NSE";
 
 -- Script Scan phases.
@@ -43,6 +49,8 @@ local BASE = "NSE_BASE";
 local WAITING_TO_RUNNING = "NSE_WAITING_TO_RUNNING";
 local DESTRUCTOR = "NSE_DESTRUCTOR";
 local SELECTED_BY_NAME = "NSE_SELECTED_BY_NAME";
+local FORMAT_TABLE = "NSE_FORMAT_TABLE";
+local FORMAT_XML = "NSE_FORMAT_XML";
 
 -- This is a limit on the number of script instance threads running at once. It
 -- exists only to limit memory use when there are many open ports. It doesn't
@@ -57,14 +65,16 @@ local NSE_SCRIPT_RULES = {
   postrule = "postrule",
 };
 
+local cnse, rules = ...; -- The NSE C library and Script Rules
+
 local _G = _G;
 
 local assert = assert;
 local collectgarbage = collectgarbage;
 local error = error;
 local ipairs = ipairs;
+local load = load;
 local loadfile = loadfile;
-local loadstring = loadstring;
 local next = next;
 local pairs = pairs;
 local pcall = pcall;
@@ -72,12 +82,10 @@ local rawget = rawget;
 local rawset = rawset;
 local require = require;
 local select = select;
-local setfenv = setfenv;
 local setmetatable = setmetatable;
 local tonumber = tonumber;
 local tostring = tostring;
 local type = type;
-local unpack = unpack;
 
 local coroutine = require "coroutine";
 local create = coroutine.create;
@@ -91,6 +99,7 @@ local traceback = debug.traceback;
 local _R = debug.getregistry();
 
 local io = require "io";
+local lines = io.lines;
 local open = io.open;
 
 local math = require "math";
@@ -112,10 +121,7 @@ local concat = table.concat;
 local insert = table.insert;
 local remove = table.remove;
 local sort = table.sort;
-
-local nmap = require "nmap";
-
-local cnse, rules = ...; -- The NSE C library and Script Rules
+local unpack = table.unpack;
 
 do -- Add loader to look in nselib/?.lua (nselib/ can be in multiple places)
   local function loader (lib)
@@ -128,18 +134,26 @@ do -- Add loader to look in nselib/?.lua (nselib/ can be in multiple places)
       return "\n\tNSE failed to find "..name.." in search paths.";
     end
   end
-  insert(package.loaders, 1, loader);
+  insert(package.searchers, 1, loader);
 end
 
+local nmap = require "nmap";
+local lfs = require "lfs";
+
+local socket = require "nmap.socket";
+local loop = socket.loop;
+
+local stdnse = require "stdnse";
+
+local strict = require "strict";
+assert(_ENV == _G);
+strict(_ENV);
+
 local script_database_type, script_database_path =
     cnse.fetchfile_absolute(cnse.script_dbpath);
 local script_database_update = cnse.scriptupdatedb;
 local script_help = cnse.scripthelp;
 
-local stdnse = require "stdnse";
-
-(require "strict")() -- strict global checking
-
 -- NSE_YIELD_VALUE
 -- This is the table C uses to yield a thread with a unique value to
 -- differentiate between yields initiated by NSE or regular coroutine yields.
@@ -205,6 +219,22 @@ local function table_size (t)
   local n = 0; for _ in pairs(t) do n = n + 1; end return n;
 end
 
+local function loadscript (filename)
+  local source = "@"..filename;
+  local function ld ()
+    -- header for scripts to allow setting the environment
+    yield [[return function (_ENV) return function (...)]];
+    -- actual script
+    for line in lines(filename, 2^15) do
+      yield(line);
+    end
+    -- footer...
+    yield [[ end end]];
+    return nil;
+  end
+  return assert(load(wrap(ld), source, "t"))();
+end
+
 -- recursively copy a table, for host/port tables
 -- not very rigorous, but it doesn't need to be
 local function tcopy (t)
@@ -231,28 +261,60 @@ rawset(stdnse, "silent_require", function (...)
   local status, mod = pcall(require, ...);
   if not status then
     print_debug(1, "%s", traceback(mod));
-    yield(REQUIRE_ERROR); -- use script yield
-    error(mod);
+    error(REQUIRE_ERROR)
   else
     return mod;
   end
 end);
 
-local Script = {}; -- The Script Class, its constructor is Script.new.
-local Thread = {}; -- The Thread Class, its constructor is Script:new_thread.
+-- Gets a string containing as much of a host's name, IP, and port as are
+-- available.
+local function against_name(host, port)
+  local targetname, ip, portno, ipport, against;
+  if host then
+    targetname = host.targetname;
+    ip = host.ip;
+  end
+  if port then
+    portno = port.number;
+  end
+  if ip and portno then
+    ipport = ip..":"..portno;
+  elseif ip then
+    ipport = ip;
+  end
+  if targetname and ipport then
+    against = targetname.." ("..ipport..")";
+  elseif targetname then
+    against = targetname;
+  elseif ipport then
+    against = ipport;
+  end
+  if against then
+    return " against "..against
+  else
+    return ""
+  end
+end
+
+-- The Script Class, its constructor is Script.new.
+local Script = {};
+-- The Thread Class, its constructor is Script:new_thread.
+local Thread = {};
+-- The Worker Class, it's a subclass of Thread. Its constructor is
+-- Thread:new_worker. It (currently) has no methods.
+local Worker = {};
 do
+  -- Workers reference data from parent thread.
+  function Worker:__index (key)
+    return Worker[key] or self.parent[key]
+  end
+
   -- Thread:d()
   -- Outputs debug information at level 1 or higher.
   -- Changes "%THREAD" with an appropriate identifier for the debug level
   function Thread:d (fmt, ...)
-    local against;
-    if self.host and self.port then
-      against = " against "..self.host.ip..":"..self.port.number;
-    elseif self.host then
-      against = " against "..self.host.ip;
-    else
-      against = "";
-    end
+    local against = against_name(self.host, self.port);
     if debugging() > 1 then
       fmt = gsub(fmt, "%%THREAD_AGAINST", self.info..against);
       fmt = gsub(fmt, "%%THREAD", self.info);
@@ -263,14 +325,29 @@ do
     print_debug(1, fmt, ...);
   end
 
-  -- Sets scripts output. Variable result is a string.
-  function Thread:set_output(result)
-    if self.type == "prerule" or self.type == "postrule" then
-      cnse.script_set_output(self.id, result);
-    elseif self.type == "hostrule" then
-      cnse.host_set_output(self.host, self.id, result);
-    elseif self.type == "portrule" then
-      cnse.port_set_output(self.host, self.port, self.id, result);
+  -- Sets script output. r1 and r2 are the (as many as two) return values.
+  function Thread:set_output(r1, r2)
+    if not self.worker then
+      -- Structure table and unstructured string outputs.
+      local tab, str
+  
+      if r2 then
+        tab, str = r1, tostring(r2);
+      elseif type(r1) == "string" then
+        tab, str = nil, r1;
+      elseif r1 == nil then
+        return
+      else
+        tab, str = r1, nil;
+      end
+  
+      if self.type == "prerule" or self.type == "postrule" then
+        cnse.script_set_output(self.id, tab, str);
+      elseif self.type == "hostrule" then
+        cnse.host_set_output(self.host, self.id, tab, str);
+      elseif self.type == "portrule" then
+        cnse.port_set_output(self.host, self.port, self.id, tab, str);
+      end
     end
   end
 
@@ -333,7 +410,7 @@ do
   function Script:new_thread (rule, ...)
     local script_type = assert(NSE_SCRIPT_RULES[rule]);
     if not self[rule] then return nil end -- No rule for this script?
-    local file_closure = self.file_closure;
+    local script_closure_generator = self.script_closure_generator;
     -- Rebuild the environment for the running thread.
     local env = {
         SCRIPT_PATH = self.filename,
@@ -341,36 +418,34 @@ do
         SCRIPT_TYPE = script_type,
     };
     setmetatable(env, {__index = _G});
-    setfenv(file_closure, env);
+    local script_closure = script_closure_generator(env);
     local unique_value = {}; -- to test valid yield
-    local function main (...)
-      file_closure(); -- loads script globals
-      return env.action(yield(unique_value, env[rule](...)));
+    local function main (_ENV, ...)
+      script_closure(); -- loads script globals
+      return action(yield(unique_value, _ENV[rule](...)));
     end
-    setfenv(main, env);
     -- This thread allows us to load the script's globals in the
     -- same Lua thread the action and rule functions will execute in.
     local co = create(main);
-    local s, value, rule_return = resume(co, ...);
-    setfenv(file_closure, _G); -- reset the environment
+    local s, value, rule_return = resume(co, env, ...);
     if s and value ~= unique_value then
       print_debug(1,
     "A thread for %s yielded unexpectedly in the file or %s function:\n%s\n",
           self.filename, rule, traceback(co));
     elseif s and (rule_return or self.forced_to_run) then
       local thread = {
+        close_handlers = {},
         co = co,
         env = env,
         identifier = tostring(co),
         info = format("'%s' (%s)", self.short_basename, tostring(co));
+        parent = nil, -- placeholder
+        script = self,
         type = script_type,
-        close_handlers = {},
+        worker = false,
       };
-      setmetatable(thread, {
-        __metatable = Thread,
-        __index = function (thread, k) return Thread[k] or self[k] end
-      }); -- Access to the parent Script
-      thread.parent = thread; -- itself
+      thread.parent = thread;
+      setmetatable(thread, Thread)
       return thread;
     elseif not s then
       log_error("A thread for %s failed to load in %s function:\n%s\n",
@@ -379,8 +454,34 @@ do
     return nil;
   end
 
+  function Thread:new_worker (main, ...)
+    local co = create(main);
+    print_debug(2, "%s spawning new thread (%s).", self.parent.info, tostring(co));
+    local thread = {
+      args = {n = select("#", ...), ...},
+      close_handlers = {},
+      co = co,
+      info = format("'%s' worker (%s)", self.short_basename, tostring(co));
+      parent = self,
+      worker = true,
+    };
+    setmetatable(thread, Worker)
+    local function info ()
+      return status(co), rawget(thread, "error");
+    end
+    return thread, info;
+  end
+
+  function Thread:resume ()
+    return resume(self.co, unpack(self.args, 1, self.args.n));
+  end
+
+  function Thread:__index (key)
+    return Thread[key] or self.script[key]
+  end
+
+  -- Script.new provides defaults for some of these.
   local required_fields = {
-    description = "string",
     action = "function",
     categories = "table",
     dependencies = "table",
@@ -423,27 +524,28 @@ do
 
     print_debug(2, "Script %s was selected by %s%s.",
         basename,
-        script_params.selection and
         script_params.selection or "(unknown)",
         script_params.forced and " and forced to run" or "");
-    local file_closure = assert(loadfile(filename));
+    local script_closure_generator = loadscript(filename);
     -- Give the closure its own environment, with global access
     local env = {
       SCRIPT_PATH = filename,
       SCRIPT_NAME = short_basename,
+      categories = {},
       dependencies = {},
     };
     setmetatable(env, {__index = _G});
-    setfenv(file_closure, env);
-    local co = create(file_closure); -- Create a garbage thread
+    local script_closure = script_closure_generator(env);
+    local co = create(script_closure); -- Create a garbage thread
     local status, e = resume(co); -- Get the globals it loads in env
     if not status then
-      log_error("Failed to load %s:\n%s", filename, traceback(co, e));
-      error("could not load script");
-    end
-    if quiet_errors[e] then
-      print_verbose(1, "Failed to load '%s'.", filename);
-      return nil;
+      if quiet_errors[e] then
+        print_verbose(1, "Failed to load '%s'.", filename);
+        return nil;
+      else
+        log_error("Failed to load %s:\n%s", filename, traceback(co, e));
+        error("could not load script");
+      end
     end
     -- Check that all the required fields were set
     for f, t in pairs(required_fields) do
@@ -484,7 +586,7 @@ do
       basename = basename,
       short_basename = short_basename,
       id = match(filename, "^.-[/\\]([^\\/]-)%.nse$") or short_basename,
-      file_closure = file_closure,
+      script_closure_generator = script_closure_generator,
       prerule = prerule,
       hostrule = hostrule,
       portrule = portrule,
@@ -500,8 +602,10 @@ do
       selected_by_name = not not script_params.verbosity,
       forced_to_run = not not script_params.forced,
     };
-    return setmetatable(script, {__index = Script, __metatable = Script});
+    return setmetatable(script, Script)
   end
+
+  Script.__index = Script;
 end
 
 -- check_rules(rules)
@@ -524,7 +628,8 @@ end
 local function get_chosen_scripts (rules)
   check_rules(rules);
 
-  local db_closure = assert(loadfile(script_database_path),
+  local db_env = {Entry = nil};
+  local db_closure = assert(loadfile(script_database_path, "t", db_env),
     "database appears to be corrupt or out of date;\n"..
     "\tplease update using: nmap --script-updatedb");
 
@@ -567,11 +672,14 @@ local function get_chosen_scripts (rules)
     local forced, rule = is_forced_set(rule);
     used_rules[rule] = false; -- has not been used yet
     forced_rules[rule] = forced;
+    -- Here we escape backslashes which might appear in Windows filenames.
+    rule = gsub(rule, "\\([^\\])", "\\\\%1");
     -- Globalize all `names`, all visible characters not ',', '(', ')', and ';'
     local globalized_rule =
         gsub(rule, "[\033-\039\042-\043\045-\058\060-\126]+", globalize);
     -- Precompile the globalized rule
-    local compiled_rule, err = loadstring("return "..globalized_rule, "rule");
+    local env = {m = nil};
+    local compiled_rule, err = load("return "..globalized_rule, "rule", "t", env);
     if not compiled_rule then
       err = err:match("rule\"]:%d+:(.+)$"); -- remove (luaL_)where in code
       error("Bad script rule:\n\t"..original_rule.." -> "..err);
@@ -580,12 +688,13 @@ local function get_chosen_scripts (rules)
     entry_rules[globalized_rule] = {
       original_rule = rule,
       compiled_rule = compiled_rule,
+      env = env,
     };
   end
 
   -- Checks if a given script, script_entry, should be loaded. A script_entry
   -- should be in the form: { filename = "name.nse", categories = { ... } }
-  local function entry (script_entry)
+  function db_env.Entry (script_entry)
     local categories, filename = script_entry.categories, script_entry.filename;
     assert(type(categories) == "table" and type(filename) == "string",
         "script database appears corrupt, try `nmap --script-updatedb`");
@@ -625,12 +734,12 @@ local function get_chosen_scripts (rules)
 
       return false;
     end
-    local env = {m = m};
 
     for globalized_rule, rule_table in pairs(entry_rules) do
       -- Clear and set the environment of the compiled script rule
-      local compiled_rule = setfenv(rule_table.compiled_rule, env)
-      local status, found = pcall(compiled_rule)
+      rule_table.env.m = m;
+      local status, found = pcall(rule_table.compiled_rule)
+      rule_table.env.m = nil;
       if not status then
         error("Bad script rule:\n\t"..rule_table.original_rule..
               " -> script rule expression not supported.");
@@ -655,7 +764,6 @@ local function get_chosen_scripts (rules)
     end
   end
 
-  setfenv(db_closure, {Entry = entry});
   db_closure(); -- Load the scripts
 
   -- Now load any scripts listed by name rather than by category.
@@ -676,11 +784,11 @@ local function get_chosen_scripts (rules)
         chosen_scripts[#chosen_scripts+1] = script;
         files_loaded[path] = true;
       elseif t == "directory" then
-        for f in cnse.dir(path) do
+        for f in lfs.dir(path) do
           local file = path .."/".. f
-          if find(f, "%.nse$") and not files_loaded[file] then
+          if find(file, "%.nse$") and not files_loaded[file] then
             script_params.selection = "directory";
-            local script = Script.new(path, script_params);
+            local script = Script.new(file, script_params);
             chosen_scripts[#chosen_scripts+1] = script;
             files_loaded[file] = true;
           end
@@ -747,7 +855,7 @@ local function run (threads_iter, hosts)
     return NSE_YIELD_VALUE; -- return NSE_YIELD_VALUE
   end
   _R[BASE] = function ()
-    return current.co;
+    return current and current.co;
   end
   -- _R[WAITING_TO_RUNNING] is called by nse_restore in nse_main.cc
   _R[WAITING_TO_RUNNING] = function (co, ...)
@@ -780,34 +888,17 @@ local function run (threads_iter, hosts)
   end
   rawset(stdnse, "new_thread", function (main, ...)
     assert(type(main) == "function", "function expected");
-    local co = create(function(...) main(...) end); -- do not return results
-    print_debug(2, "%s spawning new thread (%s).",
-        current.parent.info, tostring(co));
-    local thread = {
-      co = co,
-      id = current.id,
-      args = {n = select("#", ...), ...},
-      host = current.host,
-      port = current.port,
-      type = current.type,
-      parent = current.parent,
-      info = format("'%s' worker (%s)", current.short_basename, tostring(co));
-      close_handlers = {},
-      -- d = function(...) end, -- output no debug information
-    };
-    local thread_mt = {
-      __metatable = Thread,
-      __index = current,
-    };
-    setmetatable(thread, thread_mt);
-    total, all[co], pending[co] = total+1, thread, thread;
-    local function info ()
-      return status(co), rawget(thread, "error");
+    if current == nil then
+      error "stdnse.new_thread can only be run from an active script"
     end
-    return co, info;
+    local worker, info = current:new_worker(main, ...);
+    total, all[worker.co], pending[worker.co], num_threads = total+1, worker, worker, num_threads+1;
+    worker:start(timeouts);
+    return worker.co, info;
   end);
+
   rawset(stdnse, "base", function ()
-    return current.co;
+    return current and current.co;
   end);
 
   while threads_iter and num_threads < CONCURRENCY_LIMIT do
@@ -879,14 +970,18 @@ local function run (threads_iter, hosts)
       current, running[co] = thread, nil;
       thread:start_time_out_clock();
 
-      local s, result = resume(co, unpack(thread.args, 1, thread.args.n));
+      -- Threads may have zero, one, or two return values.
+      local s, r1, r2 = thread:resume();
       if not s then -- script error...
         all[co], num_threads = nil, num_threads-1;
-        thread:d("%THREAD_AGAINST threw an error!\n%s\n",
-            traceback(co, tostring(result)));
-        thread:close(timeouts, result);
+        if debugging() > 0 then
+          thread:d("%THREAD_AGAINST threw an error!\n%s\n", traceback(co, tostring(r1)));
+        else
+          thread:set_output("ERROR: Script execution failed (use -d to debug)");
+        end
+        thread:close(timeouts, r1);
       elseif status(co) == "suspended" then
-        if result == NSE_YIELD_VALUE then
+        if r1 == NSE_YIELD_VALUE then
           waiting[co] = thread;
         else
           all[co], num_threads = nil, num_threads-1;
@@ -895,22 +990,14 @@ local function run (threads_iter, hosts)
         end
       elseif status(co) == "dead" then
         all[co], num_threads = nil, num_threads-1;
-        if type(result) == "string" then
-          -- Escape any character outside the range 32-126 except for tab,
-          -- carriage return, and line feed. This makes the string safe for
-          -- screen display as well as XML (see section 2.2 of the XML spec).
-          result = gsub(result, "[^\t\r\n\032-\126]", function(a)
-            return format("\\x%02X", byte(a));
-          end);
-          thread:set_output(result);
-        end
+        thread:set_output(r1, r2);
         thread:d("Finished %THREAD_AGAINST.");
         thread:close(timeouts);
       end
       current = nil;
     end
 
-    cnse.nsock_loop(50); -- Allow nsock to perform any pending callbacks
+    loop(50); -- Allow nsock to perform any pending callbacks
     -- Move pending threads back to running.
     for co, thread in pairs(pending) do
       pending[co], running[co] = nil, thread;
@@ -922,6 +1009,75 @@ local function run (threads_iter, hosts)
   progress "endTask";
 end
 
+-- This function does the automatic formatting of Lua objects into strings, for
+-- normal output and for the XML @output attribute. Each nested table is
+-- indented by two spaces. Tables having a __tostring metamethod are converted
+-- using tostring. Otherwise, integer keys are listed first and only their
+-- value is shown; then string keys are shown prefixed by the key and a colon.
+-- Any other kinds of keys. Anything that is not a table is converted to a
+-- string with tostring.
+local function format_table(obj, indent)
+  indent = indent or "  ";
+  if type(obj) == "table" then
+    local mt = getmetatable(obj)
+    if mt and mt["__tostring"] then
+      -- Table obeys tostring, so use that.
+      return tostring(obj)
+    end
+
+    local lines = {};
+    -- Do integer keys.
+    for _, v in ipairs(obj) do
+      lines[#lines + 1] = indent .. format_table(v, indent .. "  ");
+    end
+    -- Do string keys.
+    for k, v in pairs(obj) do
+      if type(k) == "string" then
+        lines[#lines + 1] = indent .. k .. ": " .. format_table(v, indent .. "  ");
+      end
+    end
+    return "\n" .. concat(lines, "\n");
+  else
+    return tostring(obj);
+  end
+end
+_R[FORMAT_TABLE] = format_table
+
+local format_xml
+local function format_xml_elem(obj, key)
+  if key then
+    key = cnse.protect_xml(tostring(key));
+  end
+  if type(obj) == "table" then
+    cnse.xml_start_tag("table", {key=key});
+    cnse.xml_newline();
+  else
+    cnse.xml_start_tag("elem", {key=key});
+  end
+  format_xml(obj);
+  cnse.xml_end_tag();
+  cnse.xml_newline();
+end
+
+-- This function writes an XML representation of a Lua object to the XML stream.
+function format_xml(obj, key)
+  if type(obj) == "table" then
+    -- Do integer keys.
+    for _, v in ipairs(obj) do
+      format_xml_elem(v);
+    end
+    -- Do string keys.
+    for k, v in pairs(obj) do
+      if type(k) == "string" then
+        format_xml_elem(v, k);
+      end
+    end
+  else
+    cnse.xml_write_escaped(cnse.protect_xml(tostring(obj)));
+  end
+end
+_R[FORMAT_XML] = format_xml
+
 -- Format NSEDoc markup (e.g., including bullet lists and <code> sections) into
 -- a display string at the given indentation level. Currently this only indents
 -- the string and doesn't interpret any other markup.
@@ -942,7 +1098,9 @@ local function script_help_normal(chosen_scripts)
     log_write_raw("stdout", format("%s\n", script.id));
     log_write_raw("stdout", format("Categories: %s\n", concat(script.categories, " ")));
     log_write_raw("stdout", format("%s\n", nsedoc_url(script.id)));
-    log_write_raw("stdout", format_nsedoc(script.description, "  "));
+    if script.description then
+      log_write_raw("stdout", format_nsedoc(script.description, "  "));
+    end
   end
 end
 
@@ -975,10 +1133,12 @@ local function script_help_xml(chosen_scripts)
     cnse.xml_end_tag();
     cnse.xml_newline();
 
-    cnse.xml_start_tag("description");
-    cnse.xml_write_escaped(script.description);
-    cnse.xml_end_tag();
-    cnse.xml_newline();
+    if script.description then
+      cnse.xml_start_tag("description");
+      cnse.xml_write_escaped(script.description);
+      cnse.xml_end_tag();
+      cnse.xml_newline();
+    end
 
     -- script
     cnse.xml_end_tag();
@@ -992,6 +1152,7 @@ end
 
 do -- Load script arguments (--script-args)
   local args = cnse.scriptargs or "";
+  print_debug(1, "Script Arguments seen from CLI: %s", args);
 
   -- Parse a string in 'str' at 'start'.
   local function parse_string (str, start)
@@ -1064,6 +1225,11 @@ do -- Load script arguments (--script-args)
     end
     nmap.registry.args = tmpargs
   end
+  if debugging() >= 2 then
+    local out = {}
+    rawget(stdnse, "pretty_printer")(nmap.registry.args, function (s) out[#out+1] = s end)
+    print_debug(2, concat(out))
+  end
 end
 
 -- Update Missing Script Database?
@@ -1079,7 +1245,7 @@ if script_database_update then
   script_database_path = path.."script.db";
   local db = assert(open(script_database_path, 'w'));
   local scripts = {};
-  for f in cnse.dir(path) do
+  for f in lfs.dir(path) do
     if match(f, '%.nse$') then
       scripts[#scripts+1] = path.."/"..f;
     end
@@ -1087,13 +1253,15 @@ if script_database_update then
   sort(scripts);
   for i, script in ipairs(scripts) do
     script = Script.new(script);
-    sort(script.categories);
-    db:write('Entry { filename = "', script.basename, '", ');
-    db:write('categories = {');
-    for j, category in ipairs(script.categories) do
-      db:write(' "', lower(category), '",');
+    if ( script ) then
+      sort(script.categories);
+      db:write('Entry { filename = "', script.basename, '", ');
+      db:write('categories = {');
+      for j, category in ipairs(script.categories) do
+        db:write(' "', lower(category), '",');
+      end
+      db:write(' } }\n');
     end
-    db:write(' } }\n');
   end
   db:close();
   log_write("stdout", "Script Database updated successfully.");
@@ -1168,6 +1336,10 @@ local function main (hosts, scantype)
     print_verbose(1, "Script Post-scanning.");
   end
 
+  -- These functions do not exist until we are executing action functions.
+  rawset(stdnse, "new_thread", nil)
+  rawset(stdnse, "base", nil)
+
   for runlevel, scripts in ipairs(runlevels) do
     -- This iterator is passed to the run function. It returns one new script
     -- thread on demand until exhausted.
