@@ -5,12 +5,16 @@
 -- @author Ron Bowes <ron@skullsecurity.net>
 -- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
 
-module(... or "netbios", package.seeall)
+local bin = require "bin"
+local bit = require "bit"
+local dns = require "dns"
+local math = require "math"
+local nmap = require "nmap"
+local stdnse = require "stdnse"
+local string = require "string"
+local table = require "table"
+_ENV = stdnse.module("netbios", stdnse.seeall)
 
-require 'bit'
-require 'bin'
-require 'stdnse'
-require 'dns'
 
 types = {
 	NB = 32,
@@ -260,12 +264,27 @@ function do_nbstat(host)
 	local socket = nmap.new_socket()
 	local encoded_name = name_encode("*")
 	local statistics
+	local reg
+	if type(host) == "string" then --ip
+		stdnse.print_debug(3, "Performing nbstat on host '%s'", host)
+		nmap.registry.netbios = nmap.registry.netbios or {}
+		nmap.registry.netbios[host] = nmap.registry.netbios[host] or {}
+		reg = nmap.registry.netbios[host]
+	else
+		stdnse.print_debug(3, "Performing nbstat on host '%s'", host.ip)
+		if host.registry.netbios == nil and
+				nmap.registry.netbios ~= nil and
+				nmap.registry.netbios[host.ip] ~= nil then
+			host.registry.netbios = nmap.registry.netbios[host.ip]
+		end
+		host.registry.netbios = host.registry.netbios or {}
+		reg = host.registry.netbios
+	end
 
-	stdnse.print_debug(3, "Performing nbstat on host '%s'", host)
-	-- Check if it's cased in the registry for this host
-	if(nmap.registry["nbstat_names_" .. host] ~= nil) then
+	-- Check if it's cached in the registry for this host
+	if(reg["nbstat_names"] ~= nil) then
 		stdnse.print_debug(3, " |_ [using cached value]")
-		return true, nmap.registry["nbstat_names_" .. host], nmap.registry["nbstat_statistics_" .. host]
+		return true, reg["nbstat_names"], reg["nbstat_statistics"]
 	end
 
 	-- Create the query header
@@ -365,8 +384,8 @@ function do_nbstat(host)
 		pos, statistics = bin.unpack(string.format(">A%d", rrlength), result, pos)
 
 		-- Put it in the registry, in case anybody else needs it
-		nmap.registry["nbstat_names_"      .. host] = names
-		nmap.registry["nbstat_statistics_" .. host] = statistics
+		reg["nbstat_names"] = names
+		reg["nbstat_statistics"] = statistics
 
 		return true, names, statistics
 
@@ -441,3 +460,5 @@ function flags_to_string(flags)
 	return result
 end
 
+
+return _ENV;
