@@ -3,42 +3,47 @@
 -- functions that are too small to justify modules of their own.
 --
 -- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
+-- @class module
+-- @name stdnse
 
+local _G = require "_G"
+local coroutine = require "coroutine"
+local math = require "math"
+local nmap = require "nmap"
+local os = require "os"
+local string = require "string"
+local table = require "table"
 local assert = assert;
 local error = error;
-local pairs = pairs
+local getmetatable = getmetatable;
 local ipairs = ipairs
-local tonumber = tonumber;
-local type = type
+local pairs = pairs
+local next = next
+local rawset = rawset
+local require = require;
 local select = select
-local unpack = unpack
+local setmetatable = setmetatable;
+local tonumber = tonumber;
+local tostring = tostring;
+local type = type
 
 local ceil = math.ceil
 local max = math.max
+
 local format = string.format;
 local rep = string.rep
+
 local concat = table.concat;
 local insert = table.insert;
-local os = os
-local math = math
-local string = string
+local pack = table.pack;
+local unpack = table.unpack;
 
-local io = require 'io'; -- TODO: Remove
-
-local nmap = require "nmap";
-
-local c_funcs = require "stdnse.c";
+local difftime = os.difftime;
+local time = os.time;
 
 local EMPTY = {}; -- Empty constant table
 
-module(... or "stdnse");
-
--- Load C functions from stdnse.c into this namespace.
-for k, v in pairs(c_funcs) do
-  _M[k] = v
-end
--- Remove visibility of the stdnse.c table.
-c = nil
+_ENV = require "strict" {};
 
 --- Sleeps for a given amount of time.
 --
@@ -49,11 +54,10 @@ c = nil
 -- @class function
 -- @param t Time to sleep, in seconds.
 -- @usage stdnse.sleep(1.5)
-
--- sleep is a C function defined in nse_nmaplib.cc.
+_ENV.sleep = nmap.socket.sleep;
 
 ---
--- Prints a formatted debug message if the current verbosity level is greater
+-- Prints a formatted debug message if the current debugging level is greater
 -- than or equal to a given level.
 -- 
 -- This is a convenience wrapper around
@@ -73,6 +77,28 @@ print_debug = function(level, fmt, ...)
   end
 end
 
+---
+-- Prints a formatted verbosity message if the current verbosity level is greater
+-- than or equal to a given level.
+-- 
+-- This is a convenience wrapper around
+-- <code>nmap.log_write</code>. The first optional numeric
+-- argument, <code>level</code>, is used as the verbosity level necessary
+-- to print the message (it defaults to 1 if omitted). All remaining arguments
+-- are processed with Lua's <code>string.format</code> function.
+-- @param level Optional verbosity level.
+-- @param fmt Format string.
+-- @param ... Arguments to format.
+print_verbose = function(level, fmt, ...)
+  local l, d = tonumber(level), nmap.verbosity();
+  if l and l <= d then
+    nmap.log_write("stdout", format(fmt, ...));
+  elseif not l and 1 <= d then
+    nmap.log_write("stdout", format(level, fmt, ...));
+  end
+end
+
+
 --- Join a list of strings with a separator string.
 -- 
 -- This is Lua's <code>table.concat</code> function with the parameters
@@ -114,6 +140,30 @@ function strsplit(pattern, text)
   return list;
 end
 
+--- Generate a random string.
+-- You can either provide your own charset or the function will use
+-- a default one which is [A-Z].
+-- @param len Length of the string we want to generate.
+-- @param charset Charset that will be used to generate the string.
+-- @return A random string of length <code>len</code> consisting of
+-- characters from <code>charset</code> if one was provided, otherwise
+-- <code>charset</code> defaults to [A-Z] letters.
+function generate_random_string(len, charset)
+  local t = {}
+  local ascii_A = 65
+  local ascii_Z = 90
+  if charset then
+    for i=1,len do
+      t[i]=charset[math.random(#charset)]
+    end
+  else
+    for i=1,len do
+      t[i]=string.char(math.random(ascii_A,ascii_Z))
+    end
+  end
+  return table.concat(t)
+end
+
 --- Return a wrapper closure around a socket that buffers socket reads into
 -- chunks separated by a pattern.
 -- 
@@ -293,6 +343,7 @@ end
 -- @return A number of seconds, or <code>nil</code> followed by an error
 -- message.
 function parse_timespec(timespec)
+  if timespec == nil then return nil, "Can't parse nil timespec" end
   local n, unit, t, m
   local multipliers = {[""] = 1, s = 1, m = 60, h = 60 * 60, ms = 0.001}
 
@@ -314,6 +365,81 @@ function parse_timespec(timespec)
   return t * m
 end
 
+-- Find the offset in seconds between local time and UTC. That is, if we
+-- interpret a UTC date table as a local date table by passing it to os.time,
+-- how much must be added to the resulting integer timestamp to make it
+-- correct?
+local function utc_offset(t)
+  -- What does the calendar say locally?
+  local localtime = os.date("*t", t)
+  -- What does the calendar say in UTC?
+  local gmtime = os.date("!*t", t)
+  -- Interpret both as local calendar dates and find the difference.
+  return difftime(os.time(localtime), os.time(gmtime))
+end
+--- Convert a date table into an integer timestamp. Unlike os.time, this does
+-- not assume that the date table represents a local time. Rather, it takes an
+-- optional offset number of seconds representing the time zone, and returns
+-- the timestamp that would result using that time zone as local time. If the
+-- offset is omitted or 0, the date table is interpreted as a UTC date. For
+-- example, 4:00 UTC is the same as 5:00 UTC+1:
+-- <code>
+-- date_to_timestamp({year=1970,month=1,day=1,hour=4,min=0,sec=0})          --> 14400
+-- date_to_timestamp({year=1970,month=1,day=1,hour=4,min=0,sec=0}, 0)       --> 14400
+-- date_to_timestamp({year=1970,month=1,day=1,hour=5,min=0,sec=0}, 1*60*60) --> 14400
+-- </code>
+-- And 4:00 UTC+1 is an earlier time:
+-- <code>
+-- date_to_timestamp({year=1970,month=1,day=1,hour=4,min=0,sec=0}, 1*60*60) --> 10800
+-- </code>
+function date_to_timestamp(date, offset)
+  offset = offset or 0
+  return os.time(date) + utc_offset(os.time(date)) - offset
+end
+
+local function format_tz(offset)
+  local sign, hh, mm
+
+  if not offset then
+    return ""
+  end
+  if offset < 0 then
+    sign = "-"
+    offset = -offset
+  else
+    sign = "+"
+  end
+  -- Truncate to minutes.
+  offset = math.floor(offset / 60)
+  hh = math.floor(offset / 60)
+  mm = math.floor(math.fmod(offset, 60))
+
+  return string.format("%s%02d:%02d", sign, hh, mm)
+end
+--- Format a date and time (and optional time zone) for structured output.
+--
+-- Formatting is done according to RFC 3339 (a profile of ISO 8601), except
+-- that a time zone may be omitted to signify an unspecified local time zone.
+-- Time zones are given as an integer number of seconds from UTC. Use
+-- <code>0</code> to mark UTC itself. Formatted strings with a time zone look
+-- like this:
+-- <code>
+-- format_timestamp(os.time(), 0)       --> "2012-09-07T23:37:42+00:00"
+-- format_timestamp(os.time(), 2*60*60) --> "2012-09-07T23:37:42+02:00"
+-- </code>
+-- Without a time zone they look like this:
+-- <code>
+-- format_timestamp(os.time())          --> "2012-09-07T23:37:42"
+-- </code>
+--
+-- This function should be used for all dates emitted as part of NSE structured
+-- output.
+function format_timestamp(t, offset)
+  local tz_string = format_tz(offset)
+  offset = offset or 0
+  return os.date("!%Y-%m-%dT%H:%M:%S", t + offset) .. tz_string
+end
+
 --- Format the difference between times <code>t2</code> and <code>t1</code>
 -- into a string in one of the forms (signs may vary):
 -- * 0s
@@ -327,7 +453,7 @@ end
 function format_difftime(t2, t1)
   local d, s, sign, yeardiff
 
-  d = os.difftime(os.time(t2), os.time(t1))
+  d = difftime(time(t2), time(t1))
   if d > 0 then
     sign = "+"
   elseif d < 0 then
@@ -351,11 +477,11 @@ function format_difftime(t2, t1)
     local tmpyear = t1.year
     -- Put t1 in the same year as t2.
     t1.year = t2.year
-    d = os.difftime(os.time(t2), os.time(t1))
+    d = difftime(time(t2), time(t1))
     if d < 0 then
       -- Too far. Back off one year.
       t1.year = t2.year - 1
-      d = os.difftime(os.time(t2), os.time(t1))
+      d = difftime(time(t2), time(t1))
     end
     yeardiff = t1.year - tmpyear
     t1.year = tmpyear
@@ -457,9 +583,9 @@ local function format_output_sub(status, data, indent)
   -- Used to put 'ERROR: ' in front of all lines on error messages
   local prefix = ""
   -- Initialize the output string to blank (or, if we're at the top, add a newline)
-  local output = ""
+  local output = {}
   if(not(indent)) then
-    output = '\n'
+    insert(output, '\n')
   end
 
   if(not(status)) then
@@ -479,12 +605,18 @@ local function format_output_sub(status, data, indent)
 
   if(data['name']) then
     if(data['warning'] and nmap.debugging() > 0) then
-      output = output .. format("%s%s%s (WARNING: %s)\n", format_get_indent(indent), prefix, data['name'], data['warning'])
+      insert(output, format("%s%s%s (WARNING: %s)\n",
+                        format_get_indent(indent), prefix,
+                        data['name'], data['warning']))
     else
-      output = output .. format("%s%s%s\n", format_get_indent(indent), prefix, data['name'])
+      insert(output, format("%s%s%s\n",
+                        format_get_indent(indent), prefix,
+                        data['name']))
     end
   elseif(data['warning'] and nmap.debugging() > 0) then
-      output = output .. format("%s%s(WARNING: %s)\n", format_get_indent(indent), prefix, data['warning'])
+    insert(output, format("%s%s(WARNING: %s)\n",
+                      format_get_indent(indent), prefix,
+                      data['warning']))
   end
 
   for i, value in ipairs(data) do
@@ -501,18 +633,20 @@ local function format_output_sub(status, data, indent)
         insert(new_indent, true)
       end
 
-      output = output .. format_output_sub(status, value, new_indent)
+      insert(output, format_output_sub(status, value, new_indent))
         
     elseif(type(value) == 'string') then
       local lines = splitlines(value)
 
       for j, line in ipairs(lines) do
-        output = output .. format_get_indent(indent, i == #data and j == #lines) .. "  " .. prefix .. line .. "\n"
+      	insert(output, format("%s  %s%s\n",
+      	                  format_get_indent(indent, i == #data and j == #lines),
+      	                  prefix, line))
       end
     end
   end
 
-  return output
+  return concat(output)
 end
 
 ---Takes a table of output on the commandline and formats it for display to the 
@@ -556,7 +690,7 @@ end
 --
 --@param status A boolean value dictating whether or not the script succeeded. 
 --              If status is false, and debugging is enabled, 'ERROR' is prepended
---              to every line. If status is false and ebugging is disabled, no output
+--              to every line. If status is false and debugging is disabled, no output
 --              occurs. 
 --@param data   The table of output. 
 --@param indent Used for indentation on recursive calls; should generally be set to
@@ -588,7 +722,16 @@ end
 local function arg_value(argname)
   if nmap.registry.args[argname] then
     return nmap.registry.args[argname]
+  else
+    -- if scriptname.arg is not there, check "arg"
+    local argument_frags = strsplit("%.", argname)
+    if #argument_frags > 0 then
+      if nmap.registry.args[argument_frags[2]] then
+        return nmap.registry.args[argument_frags[2]]
+      end
+    end
   end
+
   for _, v in ipairs(nmap.registry.args) do
     if v == argname then
       return 1
@@ -721,7 +864,7 @@ end
 
 ---Similar to <code>registry_add_array</code>, except instead of adding a value to the
 -- end of an array, it adds a key:value pair to the table. 
-function registry_add_table(subkeys, key, value)
+function registry_add_table(subkeys, key, value, allow_duplicates)
   local registry = nmap.registry
   local i = 1
 
@@ -814,7 +957,7 @@ end
 --  repeat
 --    local j = math.min(i+10, #requests);
 --    local co = stdnse.new_thread(thread_main, host, port, responses,
---        unpack(requests, i, j));
+--        table.unpack(requests, i, j));
 --    threads[co] = true;
 --    i = j+1;
 --  until i > #requests;
@@ -846,3 +989,242 @@ do end -- no function here, see nse_main.lua
 --@class function
 --@return coroutine Returns the base coroutine of the running script.
 do end -- no function here, see nse_main.lua
+
+--- The Lua Require Function with errors silenced.
+--
+-- See the Lua manual for description of the require function. This modified
+-- version allows the script to quietly fail at loading if a required
+-- library does not exist.
+--
+--@name silent_require
+--@class function
+--@usage stdnse.silent_require "openssl"
+do end -- no function here, see nse_main.lua
+
+
+
+---Checks if the port is in the port range
+-- For example, calling:
+-- <code>in_port_range({number=31337,protocol="udp"},"T:15,50-75,U:31334-31339")</code>
+-- would result in a true value
+--@param port a port structure containing keys port number(number) and protocol(string)
+--@param port_range a port range string in Nmap standard format (ex. "T:80,1-30,U:31337,21-25")
+--@returns boolean indicating whether the port is in the port range
+function in_port_range(port,port_range)
+	assert(port and type(port.number)=="number" and type(port.protocol)=="string" and 
+			(port.protocol=="udp" or port.protocol=="tcp"),"Port structure missing or invalid: port={ number=<port_number>, protocol=<port_protocol> }")
+	assert((type(port_range)=="string" or type(port_range)=="number") and port_range~="","Incorrect port range specification.")
+	
+	-- Proto - true for TCP, false for UDP
+	local proto
+	if(port.protocol=="tcp") then proto = true else proto = false end
+	
+	--TCP flag for iteration - true for TCP, false for UDP, if not specified we presume TCP
+	local tcp_flag = true
+	
+	-- in case the port_range is a single number 
+	if type(port_range)=="number" then
+		if proto and port_range==port.number then return true
+		else return false
+		end
+	end
+	
+	--clean the string a bit
+	port_range=port_range:gsub("%s+","")
+	
+	-- single_pr - single port range
+	for i, single_pr in ipairs(strsplit(",",port_range)) do
+		if single_pr:match("T:") then
+			tcp_flag = true
+			single_pr = single_pr:gsub("T:","")
+		else 
+			if single_pr:match("U:") then 
+				tcp_flag = false 
+				single_pr = single_pr:gsub("U:","")
+			end
+		end
+		
+		-- compare ports only when the port's protocol is the same as
+		-- the current single port range
+		if tcp_flag == proto then
+			local pone = single_pr:match("^(%d+)$")
+			if pone then
+				pone = tonumber(pone)
+				assert(pone>-1 and pone<65536, "Port range number out of range (0-65535).")
+				
+				if pone == port.number then 
+					return true 
+				end
+			else
+				local pstart, pend = single_pr:match("^(%d+)%-(%d+)$")
+				pstart, pend = tonumber(pstart), tonumber(pend)
+				assert(pstart,"Incorrect port range specification.")
+				assert(pstart<=pend,"Incorrect port range specification, the starting port should have a smaller value than the ending port.")
+				assert(pstart>-1 and pstart<65536 and pend>-1 and pend<65536, "Port range number out of range (0-65535).")
+				
+				if port.number >=pstart and port.number <= pend then
+					return true
+				end
+			end
+		end
+	end
+	-- if no match is found then the port doesn't belong to the port_range
+	return false
+end
+
+--- Module function that mimics some behavior of Lua 5.1 module function.
+--
+-- This convenience function returns a module environment to set the _ENV
+-- upvalue. The _NAME, _PACKAGE, and _M fields are set as in the Lua 5.1
+-- version of this function. Each option function (e.g. stdnse.seeall)
+-- passed is run with the new environment, in order.
+--
+-- @see stdnse.seeall
+-- @see strict
+-- @usage
+--   _ENV = stdnse.module(name, stdnse.seeall, require "strict");
+-- @param name The module name.
+-- @param ... Option functions which modify the environment of the module.
+function module (name, ...)
+  local env = {};
+  env._NAME = name;
+  env._PACKAGE = name:match("(.+)%.[^.]+$");
+  env._M = env;
+  local mods = pack(...);
+  for i = 1, mods.n do
+    mods[i](env);
+  end
+  return env;
+end
+
+--- Change environment to load global variables.
+--
+-- Option function for use with stdnse.module. It is the same
+-- as package.seeall from Lua 5.1.
+--
+-- @see stdnse.module
+-- @usage
+--  _ENV = stdnse.module(name, stdnse.seeall);
+-- @param env Environment to change.
+function seeall (env)
+  local m = getmetatable(env) or {};
+  m.__index = _G;
+  setmetatable(env, m);
+end
+
+--- Return a table that keeps elements in order of insertion.
+--
+-- The pairs function, called on a table returned by this function, will yield
+-- elements in the order they were inserted. This function is meant to be used
+-- to construct output tables returned by scripts.
+--
+-- Reinserting a key that is already in the table does not change its position
+-- in the order. However, removing a key by assigning to <code>nil</code> and
+-- then doing another assignment will move the key to the end of the order.
+--
+-- @return An ordered table.
+function output_table ()
+  local t = {}
+  local order = {}
+  local function iterator ()
+    for i, key in ipairs(order) do
+      coroutine.yield(key, t[key])
+    end
+  end
+  local mt = {
+    __newindex = function (_, k, v)
+      if t[k] == nil and v ~= nil then
+        -- New key?
+        table.insert(order, k)
+      elseif v == nil then
+        -- Deleting an existing key?
+        for i, key in ipairs(order) do
+          if key == k then
+            table.remove(order, i)
+            break
+          end
+        end
+      end
+      rawset(t, k, v)
+    end,
+    __index = function (_, k)
+      return t[k]
+    end,
+    __pairs = function (_)
+      return coroutine.wrap(iterator)
+    end,
+    __call = function (_) -- hack to mean "not_empty?"
+      return not not next(order)
+    end
+  }
+  return setmetatable({}, mt)
+end
+
+--- A pretty printer for Lua objects.
+--
+-- Takes an object (usually a table) and prints it using the
+-- printer function. The printer function takes a sole string
+-- argument and will be called repeatedly.
+--
+-- @args obj The object to pretty print.
+-- @args printer The printer function.
+function pretty_printer (obj, printer)
+  if printer == nil then printer = print end
+
+  local function aux (obj, spacing)
+    local t = type(obj)
+    if t == "table" then
+      printer "{\n"
+      for k, v in pairs(obj) do
+        local spacing = spacing.."\t"
+        printer(spacing)
+        printer "["
+        aux(k, spacing)
+        printer "] = "
+        aux(v, spacing)
+        printer ",\n"
+      end
+      printer(spacing.."}")
+    elseif t == "string" then
+      printer(format("%q", obj))
+    else
+      printer(tostring(obj))
+    end
+  end
+
+  return aux(obj, "")
+end
+
+-- This pattern must match the percent sign '%' since it is used in
+-- escaping.
+local FILESYSTEM_UNSAFE = "[^a-zA-Z0-9._-]"
+---
+-- Escape a string to remove bytes and strings that may have meaning to
+-- a filesystem, such as slashes. All bytes are escaped, except for:
+-- * alphabetic <code>a</code>-<code>z</code> and <code>A</code>-<code>Z</code>, digits 0-9, <code>.</code> <code>_</code> <code>-</code>
+-- In addition, the strings <code>"."</code> and <code>".."</code> have
+-- their characters escaped.
+--
+-- Bytes are escaped by a percent sign followed by the two-digit
+-- hexadecimal representation of the byte value.
+-- * <code>filename_escape("filename.ext") --> "filename.ext"</code>
+-- * <code>filename_escape("input/output") --> "input%2foutput"</code>
+-- * <code>filename_escape(".") --> "%2e"</code>
+-- * <code>filename_escape("..") --> "%2e%2e"</code>
+-- This escaping is somewhat like that of JavaScript
+-- <code>encodeURIComponent</code>, except that fewer bytes are
+-- whitelisted, and it works on bytes, not Unicode characters or UTF-16
+-- code points.
+function filename_escape(s)
+  if s == "." then
+    return "%2e"
+  elseif s == ".." then
+    return "%2e%2e"
+  else
+    return (string.gsub(s, FILESYSTEM_UNSAFE, function (c)
+      return string.format("%%%02x", string.byte(c))
+    end))
+  end
+end
+
+return _ENV;
