@@ -1,11 +1,10 @@
-
 /***************************************************************************
  * error.h -- a few simple routines for dealing with errors (quitting,     *
  * printing error messages, etc.                                           *
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2011 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2013 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -34,17 +33,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -54,7 +54,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: error.h 21905 2011-01-21 00:04:51Z fyodor $ */
+/* $Id: error.h 31563 2013-07-28 22:08:48Z fyodor $ */
 
 #ifndef ERROR_H
 #define ERROR_H
@@ -75,11 +75,15 @@
 #include <unistd.h>
 #endif
 
-void fatal(char *fmt, ...) __attribute__((noreturn));
-void myerror(char *fmt, ...);
-void pfatal(char *fmt, ...) __attribute__((noreturn));
+void fatal(char *fmt, ...)
+  __attribute__ ((noreturn))
+  __attribute__ ((format (printf, 1, 2)));
+
+void pfatal(char *fmt, ...)
+  __attribute__ ((noreturn))
+  __attribute__ ((format (printf, 1, 2)));
+
 void gh_perror(char *err, ...);
 
 #endif /* ERROR_H */
 
-
