@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2012 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2013 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -35,17 +35,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -55,7 +56,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_engines.c 28195 2012-03-01 09:05:02Z henri $ */
+/* $Id: nsock_engines.c 31563 2013-07-28 22:08:48Z fyodor $ */
 
 #ifdef HAVE_CONFIG_H
 #include "nsock_config.h"
@@ -71,6 +72,20 @@
   #define ENGINE_EPOLL
 #endif /* HAVE_EPOLL */
 
+#if HAVE_KQUEUE
+  extern struct io_engine engine_kqueue;
+  #define ENGINE_KQUEUE &engine_kqueue,
+#else
+  #define ENGINE_KQUEUE
+#endif /* HAVE_KQUEUE */
+
+#if HAVE_POLL
+  extern struct io_engine engine_poll;
+  #define ENGINE_POLL &engine_poll,
+#else
+  #define ENGINE_POLL
+#endif /* HAVE_POLL */
+
 /* select() based engine is the fallback engine, we assume it's always available */
 extern struct io_engine engine_select;
 #define ENGINE_SELECT &engine_select,
@@ -79,6 +94,8 @@ extern struct io_engine engine_select;
  * available on your system. Engines must be sorted by order of preference */
 static struct io_engine *available_engines[] = {
   ENGINE_EPOLL
+  ENGINE_KQUEUE
+  ENGINE_POLL
   ENGINE_SELECT
   NULL
 };
@@ -107,13 +124,38 @@ struct io_engine *get_io_engine(void) {
   return engine;
 }
 
-void nsock_set_default_engine(char *engine) {
+int nsock_set_default_engine(char *engine) {
   if (engine_hint)
     free(engine_hint);
 
-  if (engine)
-    engine_hint = strdup(engine);
-  else
-    engine_hint = NULL;
+  if (engine) {
+    int i;
+
+    for (i = 0; available_engines[i] != NULL; i++) {
+      if (strcmp(engine, available_engines[i]->name) == 0) {
+        engine_hint = strdup(engine);
+        return 0;
+      }
+    }
+    return -1;
+  }
+  /* having engine = NULL is fine. This is actually the
+   * way to tell nsock to use the default engine again. */
+  engine_hint = NULL;
+  return 0;
+}
+
+const char *nsock_list_engines(void) {
+  return
+#if HAVE_EPOLL
+  "epoll "
+#endif
+#if HAVE_KQUEUE
+  "kqueue "
+#endif
+#if HAVE_POLL
+  "poll "
+#endif
+  "select";
 }
 
