@@ -5,59 +5,91 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
+ * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
+ * modify, and redistribute this software under certain conditions.  If    *
+ * you wish to embed Nmap technology into proprietary software, we sell    *
+ * alternative licenses (contact sales@insecure.com).  Dozens of software  *
+ * vendors already license Nmap technology such as host discovery, port    *
+ * scanning, OS detection, version detection, and the Nmap Scripting       *
+ * Engine.                                                                 *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
  * misunderstandings, we interpret that term as broadly as copyright law   *
  * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
+ *                                                                         *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two.                                  *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the special and conditions of the license text as well.       *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, are happy to help.  As mentioned above, we also    *
+ * offer alternative license to integrate Nmap into proprietary            *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
  * as providing for priority support and updates.  They also fund the      *
  * continued development of Nmap.  Please email sales@insecure.com for     *
  * further information.                                                    *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
- *                                                                         *
  * If you received these files with a written license agreement or         *
  * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
@@ -69,8 +101,8 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
  * source code repository, it is understood (unless you specify otherwise) *
  * that you are offering the Nmap Project (Insecure.Com LLC) the           *
@@ -84,14 +116,13 @@
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: services.cc 28192 2012-03-01 06:53:35Z fyodor $ */
+/* $Id: services.cc 31563 2013-07-28 22:08:48Z fyodor $ */
 
 #include "nmap.h"
 #include "services.h"
@@ -106,7 +137,7 @@
 /* This structure is the key for looking up services in the
    port/proto -> service map. */
 struct port_spec {
-  int portno; /* Network byte order */
+  int portno;
   std::string proto;
 
   /* Sort in the usual nmap-services order. */
@@ -234,8 +265,6 @@ static int nmap_services_init() {
       continue;
     }
 
-    portno = htons(portno);
-
     port_spec ps;
     ps.portno = portno;
     ps.proto = proto;
@@ -245,7 +274,7 @@ static int nmap_services_init() {
     i = service_table.find(ps);
     if (i != service_table.end()) {
       if (o.debugging)
-        error("Port %d proto %s is duplicated in services file %s", ntohs(portno), proto, filename);
+        error("Port %d proto %s is duplicated in services file %s", portno, proto, filename);
       continue;
     }
 
@@ -314,15 +343,15 @@ int addportsfromservmask(char *mask, u8 *porttbl, int range_type) {
     service_node& current = i->second;
     if (wildtest(mask, current.s_name)) {
       if ((range_type & SCAN_TCP_PORT) && strcmp(current.s_proto, "tcp") == 0) {
-        porttbl[ntohs(current.s_port)] |= SCAN_TCP_PORT;
+        porttbl[current.s_port] |= SCAN_TCP_PORT;
         t++;
       }
       if ((range_type & SCAN_UDP_PORT) && strcmp(current.s_proto, "udp") == 0) {
-        porttbl[ntohs(current.s_port)] |= SCAN_UDP_PORT;
+        porttbl[current.s_port] |= SCAN_UDP_PORT;
         t++;
       }
       if ((range_type & SCAN_SCTP_PORT) && strcmp(current.s_proto, "sctp") == 0) {
-        porttbl[ntohs(current.s_port)] |= SCAN_SCTP_PORT;
+        porttbl[current.s_port] |= SCAN_SCTP_PORT;
         t++;
       }
     }
@@ -333,7 +362,6 @@ int addportsfromservmask(char *mask, u8 *porttbl, int range_type) {
 
 
 
-/* Port must be in network byte order. */
 struct servent *nmap_getservbyport(int port, const char *proto) {
   std::map<port_spec, service_node>::iterator i;
   port_spec ps;
@@ -371,15 +399,15 @@ static bool is_port_member(const struct scan_lists *ptsdata, const struct servic
 
   if (strcmp(serv->s_proto, "tcp") == 0) {
     for (i=0; i<ptsdata->tcp_count; i++)
-      if (ntohs(serv->s_port) == ptsdata->tcp_ports[i])
+      if (serv->s_port == ptsdata->tcp_ports[i])
         return true;
   } else if (strcmp(serv->s_proto, "udp") == 0) {
     for (i=0; i<ptsdata->udp_count; i++)
-      if (ntohs(serv->s_port) == ptsdata->udp_ports[i])
+      if (serv->s_port == ptsdata->udp_ports[i])
         return true;
   } else if (strcmp(serv->s_proto, "sctp") == 0) {
     for (i=0; i<ptsdata->sctp_count; i++)
-      if (ntohs(serv->s_port) == ptsdata->sctp_ports[i])
+      if (serv->s_port == ptsdata->sctp_ports[i])
         return true;
   }
 
@@ -473,11 +501,11 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
         continue;
       if (current->ratio >= level) {
         if (o.TCPScan() && strcmp(current->s_proto, "tcp") == 0)
-          ports->tcp_ports[ti++] = ntohs(current->s_port);
+          ports->tcp_ports[ti++] = current->s_port;
         else if (o.UDPScan() && strcmp(current->s_proto, "udp") == 0)
-          ports->udp_ports[ui++] = ntohs(current->s_port);
+          ports->udp_ports[ui++] = current->s_port;
         else if (o.SCTPScan() && strcmp(current->s_proto, "sctp") == 0)
-          ports->sctp_ports[si++] = ntohs(current->s_port);
+          ports->sctp_ports[si++] = current->s_port;
       } else {
         break;
       }
@@ -506,11 +534,11 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
       if (ptsdata_initialized && !is_port_member(&ptsdata, current))
         continue;
       if (o.TCPScan() && strcmp(current->s_proto, "tcp") == 0 && ti < ports->tcp_count)
-        ports->tcp_ports[ti++] = ntohs(current->s_port);
+        ports->tcp_ports[ti++] = current->s_port;
       else if (o.UDPScan() && strcmp(current->s_proto, "udp") == 0 && ui < ports->udp_count)
-        ports->udp_ports[ui++] = ntohs(current->s_port);
+        ports->udp_ports[ui++] = current->s_port;
       else if (o.SCTPScan() && strcmp(current->s_proto, "sctp") == 0 && si < ports->sctp_count)
-        ports->sctp_ports[si++] = ntohs(current->s_port);
+        ports->sctp_ports[si++] = current->s_port;
     }
 
     if (ti < ports->tcp_count) ports->tcp_count = ti;
